@@ -12,6 +12,8 @@ from typing import Callable
 
 from terminaltables import PorcelainTable
 
+from linodecli.plugins import inherit_plugin_args
+
 BOLD = "\033[1m"
 NOT_BOLD = "\033[0m"
 
@@ -573,8 +575,11 @@ def call(args, context):
     """
     Invokes the Interactive Firewall Plugin
     """
-    parser = argparse.ArgumentParser("firewall-editor", add_help=True)
+    parser = inherit_plugin_args(
+        argparse.ArgumentParser("firewall-editor", add_help=True)
+    )
     parser.add_argument("firewall_id", help="The ID of the firewall to edit.")
+
     parsed = parser.parse_args(args)
 
     # fetch the info and start going
